@@ -5,11 +5,12 @@ Dynamics simulation module.
 """
 
 
+from .base import *
 from .neurons import *
 from .synapses import *
 from .channels import *
-from .base import *
+from .others import *
 from .utils import *
 from .runners import *
 
-from . import neurons, synapses, channels, rates, utils, runners
+from . import neurons, synapses, channels, rates, others, utils, runners
